@@ -471,6 +471,7 @@ static chunk *chunkqueue_get_append_tempfile(chunkqueue *cq) {
 			if (-1 != (fd = mkstemp(template->ptr))) break;
 		}
 	} else {
+		/* coverity[secure_temp : FALSE] */
 		fd = mkstemp(template->ptr);
 	}
 
@@ -547,6 +548,8 @@ int chunkqueue_append_mem_to_tempfile(server *srv, chunkqueue *dest, const char
 			return -1;
 		}
 
+		/* (dst_c->file.fd >= 0) */
+		/* coverity[negative_returns : FALSE] */
 		written = write(dst_c->file.fd, mem, len);
 
 		if ((size_t) written == len) {
@@ -585,14 +588,14 @@ int chunkqueue_append_mem_to_tempfile(server *srv, chunkqueue *dest, const char
 					return -1;
 				}
 			}
-			if (!retry) return -1;
+			if (!retry) break; /* return -1; */
 
 			/* continue; retry */
 		}
 
 	} while (dst_c);
 
-	return -1; /*(not reached)*/
+	return -1;
 }
 
 int chunkqueue_steal_with_tempfiles(server *srv, chunkqueue *dest, chunkqueue *src, off_t len) {
