@@ -3,6 +3,7 @@
 #include "base.h"
 #include "log.h"
 #include "buffer.h"
+#include "rand.h"
 
 #include "plugin.h"
 
@@ -230,7 +231,7 @@ URIHANDLER_FUNC(mod_usertrack_uri_handler) {
 	li_itostrn(hh, sizeof(hh), srv->cur_ts);
 	li_MD5_Update(&Md5Ctx, (unsigned char *)hh, strlen(hh));
 	li_MD5_Update(&Md5Ctx, (unsigned char *)srv->entropy, sizeof(srv->entropy));
-	li_itostrn(hh, sizeof(hh), rand());
+	li_itostrn(hh, sizeof(hh), li_rand());
 	li_MD5_Update(&Md5Ctx, (unsigned char *)hh, strlen(hh));
 
 	li_MD5_Final(h, &Md5Ctx);
