@@ -129,60 +129,74 @@ int http_response_write_header(server *srv, connection *con) {
 }
 
 #ifdef USE_OPENSSL
-static void https_add_ssl_entries(connection *con) {
+#include <openssl/bn.h>
+#include <openssl/err.h>
+static void https_add_ssl_client_entries(server *srv, connection *con) {
 	X509 *xs;
 	X509_NAME *xn;
-	X509_NAME_ENTRY *xe;
 	int i, nentries;
 
-	if (
-		SSL_get_verify_result(con->ssl) != X509_V_OK
-		|| !(xs = SSL_get_peer_certificate(con->ssl))
-	) {
+	long vr = SSL_get_verify_result(con->ssl);
+	if (vr != X509_V_OK) {
+		char errstr[256];
+		ERR_error_string_n(vr, errstr, sizeof(errstr));
+		buffer_copy_string_len(srv->tmp_buf, CONST_STR_LEN("FAILED:"));
+		buffer_append_string(srv->tmp_buf, errstr);
+		array_set_key_value(con->environment,
+				    CONST_STR_LEN("SSL_CLIENT_VERIFY"),
+				    CONST_BUF_LEN(srv->tmp_buf));
 		return;
+	} else if (!(xs = SSL_get_peer_certificate(con->ssl))) {
+		array_set_key_value(con->environment,
+				    CONST_STR_LEN("SSL_CLIENT_VERIFY"),
+				    CONST_STR_LEN("NONE"));
+		return;
+	} else {
+		array_set_key_value(con->environment,
+				    CONST_STR_LEN("SSL_CLIENT_VERIFY"),
+				    CONST_STR_LEN("SUCCESS"));
 	}
 
+	buffer_copy_string_len(srv->tmp_buf, CONST_STR_LEN("SSL_CLIENT_S_DN_"));
 	xn = X509_get_subject_name(xs);
 	for (i = 0, nentries = X509_NAME_entry_count(xn); i < nentries; ++i) {
 		int xobjnid;
 		const char * xobjsn;
-		data_string *envds;
+		X509_NAME_ENTRY *xe;
 
 		if (!(xe = X509_NAME_get_entry(xn, i))) {
 			continue;
 		}
 		xobjnid = OBJ_obj2nid((ASN1_OBJECT*)X509_NAME_ENTRY_get_object(xe));
 		xobjsn = OBJ_nid2sn(xobjnid);
-		if (!xobjsn) {
-			continue;
+		if (xobjsn) {
+			buffer_string_set_length(srv->tmp_buf, sizeof("SSL_CLIENT_S_DN_")-1);
+			buffer_append_string(srv->tmp_buf, xobjsn);
+			array_set_key_value(con->environment,
+					    CONST_BUF_LEN(srv->tmp_buf),
+					    (const char *)X509_NAME_ENTRY_get_data(xe)->data,
+					    X509_NAME_ENTRY_get_data(xe)->length);
 		}
+	}
 
-		if (NULL == (envds = (data_string *)array_get_unused_element(con->environment, TYPE_STRING))) {
-			envds = data_string_init();
-		}
-		buffer_copy_string_len(envds->key, CONST_STR_LEN("SSL_CLIENT_S_DN_"));
-		buffer_append_string(envds->key, xobjsn);
-		buffer_copy_string_len(
-			envds->value,
-			(const char *)X509_NAME_ENTRY_get_data(xe)->data,
-			X509_NAME_ENTRY_get_data(xe)->length
-		);
+	{
+		ASN1_INTEGER *xsn = X509_get_serialNumber(xs);
+		BIGNUM serialBN;
+		char *serialHex = BN_bn2hex(ASN1_INTEGER_to_BN(xsn, &serialBN));
+		array_set_key_value(con->environment,
+				    CONST_STR_LEN("SSL_CLIENT_M_SERIAL"),
+				    serialHex, strlen(serialHex));
+		OPENSSL_free(serialHex);
+	}
+
+	if (!buffer_string_is_empty(con->conf.ssl_verifyclient_username)) {
 		/* pick one of the exported values as "REMOTE_USER", for example
 		 * ssl.verifyclient.username   = "SSL_CLIENT_S_DN_UID" or "SSL_CLIENT_S_DN_emailAddress"
 		 */
-		if (buffer_is_equal(con->conf.ssl_verifyclient_username, envds->key)) {
-			data_string *ds;
-			if (NULL == (ds = (data_string *)array_get_element(con->environment, "REMOTE_USER"))) {
-				if (NULL == (ds = (data_string *)array_get_unused_element(con->environment, TYPE_STRING))) {
-					ds = data_string_init();
-				}
-				buffer_copy_string(ds->key, "REMOTE_USER");
-				array_insert_unique(con->environment, (data_unset *)ds);
-			}
-			buffer_copy_buffer(ds->value, envds->value);
-		}
-		array_insert_unique(con->environment, (data_unset *)envds);
+		data_string *ds = (data_string *)array_get_element(con->environment, con->conf.ssl_verifyclient_username->ptr);
+		if (ds) array_set_key_value(con->environment, CONST_STR_LEN("REMOTE_USER"), CONST_BUF_LEN(ds->value));
 	}
+
 	if (con->conf.ssl_verifyclient_export_cert) {
 		BIO *bio;
 		if (NULL != (bio = BIO_new(BIO_s_mem()))) {
@@ -201,11 +215,42 @@ static void https_add_ssl_entries(connection *con) {
 			BIO_read(bio, envds->value->ptr, n);
 			BIO_free(bio);
 			buffer_commit(envds->value, n);
-			array_insert_unique(con->environment, (data_unset *)envds);
+			array_replace(con->environment, (data_unset *)envds);
 		}
 	}
 	X509_free(xs);
 }
+
+void http_cgi_ssl_env(server *srv, connection *con) {
+    const char *s;
+    const SSL_CIPHER *cipher;
+    UNUSED(srv);
+
+    if (!con->ssl) return;
+
+    s = SSL_get_version(con->ssl);
+    array_set_key_value(con->environment,
+                        CONST_STR_LEN("SSL_PROTOCOL"),
+                        s, strlen(s));
+
+    if ((cipher = SSL_get_current_cipher(con->ssl))) {
+        int usekeysize, algkeysize;
+        char buf[LI_ITOSTRING_LENGTH];
+        s = SSL_CIPHER_get_name(cipher);
+        array_set_key_value(con->environment,
+                            CONST_STR_LEN("SSL_CIPHER"),
+                            s, strlen(s));
+        usekeysize = SSL_CIPHER_get_bits(cipher, &algkeysize);
+        li_itostrn(buf, sizeof(buf), usekeysize);
+        array_set_key_value(con->environment,
+                            CONST_STR_LEN("SSL_CIPHER_USEKEYSIZE"),
+                            buf, strlen(buf));
+        li_itostrn(buf, sizeof(buf), algkeysize);
+        array_set_key_value(con->environment,
+                            CONST_STR_LEN("SSL_CIPHER_ALGKEYSIZE"),
+                            buf, strlen(buf));
+    }
+}
 #endif
 
 
@@ -322,7 +367,7 @@ handler_t http_response_prepare(server *srv, connection *con) {
 
 #ifdef USE_OPENSSL
 		if (con->srv_socket->is_ssl && con->conf.ssl_verifyclient) {
-			https_add_ssl_entries(con);
+			https_add_ssl_client_entries(srv, con);
 		}
 #endif
 
@@ -517,6 +562,7 @@ handler_t http_response_prepare(server *srv, connection *con) {
 		buffer_append_slash(con->physical.path);
 		if (!buffer_string_is_empty(con->physical.rel_path) &&
 		    con->physical.rel_path->ptr[0] == '/') {
+			/* coverity[overflow_sink : FALSE] */
 			buffer_append_string_len(con->physical.path, con->physical.rel_path->ptr + 1, buffer_string_length(con->physical.rel_path) - 1);
 		} else {
 			buffer_append_string_buffer(con->physical.path, con->physical.rel_path);
